import os
import pkg_resources
from os.path import abspath


def get_fn_information(opt_input: str):
    """ takes options.input and returns a list of useful fn info """
    fn_name_only = opt_input.split("/")[-1].split('.')[0]
    return opt_input, fn_name_only


def get_output_information(options, fn_name_only):
    """ takes options.output and returns output filename """
    if options.output:
        if not options.output[-4:] == '.pdf':
            print('Error: Your output filetype must be ".pdf"')
            exit()
        else:
            output_fn = abspath(options.output)
    elif options.output_dir:
        if options.output_dir[-1] == '/':
            output_fn = f'/{options.output_dir}{fn_name_only}.pdf'
        else:
            output_fn = f'/{options.output_dir}/{fn_name_only}.pdf'
    else:
        print("Using default output destination...")
        output_fn = os.getcwd() + f'/{fn_name_only}.pdf'

    # get build directory
    if options.buildfile_dir:
        buildfiles_dir = abspath(options.buildfile_dir)
    else:
        buildfiles_dir = os.getcwd() + f'/{fn_name_only}-src'
    if not os.path.isdir(buildfiles_dir):
        # no longer cleaning b/c it'll just overwrite anyway
        os.mkdir(buildfiles_dir)

    output_html = f'{buildfiles_dir}/{fn_name_only}-buildfile.html'

    return output_fn, buildfiles_dir, output_html


def get_stylesheet_from_options(options):
    """ get the stylesheet or return default """
    if options.stylesheet:
        css = abspath(options.stylesheet)
    else:  # defaults
        print("Using default stylesheet...")
        css = pkg_resources.resource_filename('lwm2pdf',
                                              'themes/manuscript.css')
    return css



import os
import subprocess
import shutil
from weasyprint import HTML, CSS  # type: ignore
from weasyprint.fonts import FontConfiguration  # type: ignore


def open_pdf(output_fn, ask):
    """
    helper function to try and open the pdf; in theory cross-platform
    but I haven't really done much testing on this.
    """
    if ask == 'y':
        ask_to_open = 'y'
    elif ask == 'n':
        ask_to_open = "n"
    else:
        ask_to_open = input('Do you want to open the PDF? [y/n] ')
    if ask_to_open == 'y':
        try:
            try_mac = subprocess.run(['open', output_fn],
                                     capture_output=True, text=True)
            if try_mac.stderr == '':
                return try_mac.stdout
        except FileNotFoundError:
            try:
                try_linux = subprocess.run(['xdg-open', output_fn],
                                           capture_output=True, text=True)
                if try_linux.stderr == '':
                    return try_linux.stdout
            except FileNotFoundError:
                try:
                    try_pc = subprocess.run(['open', output_fn],
                                            capture_output=True, text=True)
                    if try_pc.stderr == '':
                        return try_pc.stdout
                except Exception:
                    print("Sorry, we can't seem to open the file." +
                          "Try opening with your file browser.")


def create_pdf(fn, output_fn, buildfiles_dir, output_html, styles, options):
    """ Build final PDF """
    try:
        font_config = FontConfiguration()
        HTML(output_html).write_pdf(
            output_fn,
            stylesheets=[CSS(string=styles)],
            font_config=font_config)
        # Let the user know where it lives now
        print(f"\nSuccess! A PDF from {fn} has been successfully built" +
              f" and saved to:\n{output_fn}\n")
    except AttributeError as ae:
        print(ae)
    except Exception as unk_e:
        print(f"There was an error building the PDF.\n{Exception}\n{unk_e}")

    # Cleanup build files if they're not wanted
    if not options.save_buildfile and os.path.isdir(buildfiles_dir):
        shutil.rmtree(buildfiles_dir)

    return True



import argparse


def get_lwm2pdf_options(supported_file_types: str, argv=None):
    """
    main app options parsing
    """
    supported_file_types = ""
    desc_str = f'Converts {supported_file_types} into PDFs based on' + \
               'default or user stylesheet (css).'

    cli_options_parser = argparse.ArgumentParser(prog='lwm2pdf',
                                                 usage='%(prog)s [options]',
                                                 description=desc_str)

    cli_options_parser.add_argument('-i', '--input',
                                    dest='input',
                                    action='store',
                                    type=str,
                                    required=True,
                                    help='the file to convert' +
                                         f'({supported_file_types})')

    cli_options_parser.add_argument('-o', '--output',
                                    dest='output',
                                    action='store',
                                    type=str,
                                    required=False,
                                    help='output filename and destination' +
                                         '(optional)')

    cli_options_parser.add_argument('-od', '--output-dir',
                                    dest='output_dir',
                                    action='store',
                                    type=str,
                                    required=False,
                                    help='output filename and destination' +
                                         '(optional)')

    cli_options_parser.add_argument('-s', '--stylesheet',
                                    dest='stylesheet',
                                    action='store',
                                    type=str,
                                    required=False,
                                    help='select user stylesheet (css)' +
                                         '(optional)')

    cli_options_parser.add_argument('-p', '--preserve-buildfiles',
                                    dest="save_buildfile",
                                    action="store_true",
                                    required=False,
                                    help='preserve buildfiles in output/src' +
                                         'in current working directory')

    cli_options_parser.add_argument('-bd', '--buildfile-dir',
                                    dest="buildfile_dir",
                                    action='store',
                                    type=str,
                                    required=False,
                                    help='destination for buildfile(s)' +
                                         'directory')

    cli_options_parser.add_argument('--open',
                                    action='store',
                                    type=str,
                                    required=False,
                                    dest='ask_to_open')

    return cli_options_parser.parse_args(argv)



import subprocess
import re
import markdown2  # type: ignore


def asciidoc_to_html(fn):
    """ takes an asciidoc-formatted file and returns html """
    print("Running our input through asciidoc conversion....")
    print("Checking for asciidoctor...")
    try:
        test_asciidoctor = subprocess.run(['asciidoctor', '-v'],
                                          capture_output=True,
                                          text=True)
        if test_asciidoctor.stderr == '':
            print("Asciidoctor is present; converting with asciidoctor...")
            # make a temp preformatted file
            result = subprocess.run(['asciidoctor', '-a', "stylesheet!",
                                     "-o", "-", fn],
                                    capture_output=True,
                                    text=True)

            if result.stderr == '':
                return result.stdout

            else:
                print(f'\nError: {result.stderr}')
                print("Exiting...")
                exit()

    except FileNotFoundError:
        print("\n---\nWARNING: Asciidoctor was not found on your PATH.")
        print("\nWe will proceed with the asciidoc-py converter,")
        print("but we recommend installing asciidoctor for")
        print("best results. Some features may not be available.")
        print("See https://asciidoctor.org/ for more information.\n---")
        print("\nProceeding with asciidoc-py...")

        result = subprocess.run(['asciidoc', '-b', 'html5', '-a', 'linkcss',
                                 '-a', 'disable-javascript', "-o", "-", fn],
                                capture_output=True,
                                text=True)
        if result.returncode == 0:
            html = result.stdout
            # remove linked stylesheet....
            html = html.replace('<link rel="stylesheet" href="./asciidoc.css" type="text/css">', '')
            return html

        else:
            print()
            print(f'Error code {result.returncode}: {result.stderr}')
            exit()


def md_to_html(fn):
    """ takes a markdown file and returns html """

    print("Running input through markdown conversion....")
    text = open(fn, 'r').read()
    html = markdown2.markdown(text, extras=["fenced-code-blocks",
                                            "header-ids",
                                            "footnotes",
                                            "smarty-pants"  # to save a step
                                            ])
    return html


def markup_to_html(fn, supported_file_types):
    # handle asciidoc
    if fn.find('.adoc') > -1 or fn.find('.asciidoc') > 1:
        html = asciidoc_to_html(fn)
    
    # handle markdown
    elif fn.find('.md') > -1:
        html = md_to_html(fn)
    
    else:
        html = None
        SystemExit("Error: It appears you're trying to convert an " +
                   "unsupported file format. This script accepts only " +
                   f"{supported_file_types} files.")
    return html



import re


def clean_html(html: str, section_break_marker: str = '#') -> str:
    """
    A clean function to make the html nicer for weasyprinting
    """
    # split the author info
    au_r = re.compile(r'<span id="author" class="author">(.*?)</span><br>')
    html = re.sub(au_r, split_author_names, html)

    # just swap hrs for breaks (will help with copy-paste to word)
    html = html.replace("<hr>",
            f"<div class=\"section-break\"><p>{section_break_marker}</p></div>")
    html = html.replace("<hr />",  # markdown converted handle
            f"<div class=\"section-break\"><p>{section_break_marker}</p></div>")

    # if quotes, fix citation style
    print("Cleaning up blockquotes...")
    quote_r = re.compile(r'<div class="attribution">\n&#8212; (.*?)<br>')
    html = re.sub(quote_r, swap_br_for_comma, html)

    # expand links (and handle cross references)
    # NOTE: This does not super handle classes on xref a tags...
    print("Expanding links and fixing xrefs for print...")
    link_r = re.compile(r'<a href="(.*?)">(.*?)</a>')
    html = re.sub(link_r, expand_links_and_xrefs, html)

    # fix missing xrefs
    xrefs = re.compile(r'<a href="(.*?)" data-type="xref">(.*?)</a>')
    missing_xref_string = '<a href="#" class="missing-xref">???</a>'
    for m in re.finditer(xrefs, html):
        # if the id doesn't exist
        id = m.group(1)[1:]
        if html.find(f'id="{id}') == -1:
            print(f'\nWARNING: Missing xref to #{id}\n')
            html = html.replace(m.group(0), missing_xref_string)

    # do footnotes
    print("Fixing footnotes for print...")
    html = clean_footnotes(html)

    return html


def clean_footnotes(html):
    """ clean up footnotes """
    footnote_r = re.compile(r'<sup class="footnote">\[(.*?)\]</sup>')
    html = re.sub(footnote_r, fix_footnotes, html)

    # asciidoc
    endnotes_r = "<div id=\"footnotes\">\n<div class='section-break'><p>#</p></div>"
    html = html.replace(endnotes_r, 
                        '<div id="footnotes">\n<h2>Notes</h2>')
    # markdown
    endnotes_r_md = '<div class="footnotes">\n<div class=\'section-break\'><p>#</p>'
    html = html.replace(endnotes_r_md, 
                        '<div class="footnotes">\n<h2>Notes</h2>')
    return html


# Match functions
def split_author_names(match):
    """ if possible, split author name """
    name = match.group(1).split(' ')
    # edge case
    if len(name) > 2:
        first = name[0]
        last = name[-1]
    else:
        first = name[0]
        last = name[-1]
    return f"<span id='authorFirstName'>{first}</span>&nbsp;" + \
           f"<span id='authorLastName'>{last}</span>"


def swap_br_for_comma(match):
    """ fix for asciidoc(tor) attributions """
    str = match.group(1)
    newstr = f'<div class="attribution">&#8212; {str}, '
    return newstr


def expand_links_and_xrefs(match):
    """
    since this is print, expand links such that they appear after anchored text
    """
    if not match.group(1).find("#") > -1:
        expanded_link = f'{match.group(2)} (<a href="{match.group(1)}">{match.group(1)}</a>)'
        return expanded_link

    # do xrefs but not footnotes... assuming all non-footnotes are xrefs
    elif (
        match.group(0).find("class=\"footnote") == -1 and
        match.group(0).find("href=\"#fn-") == -1 and
        match.group(0).find("href=\"#_footnoteref_") == -1
       ):
        xref = f'<a href="{match.group(1)}" data-type="xref">{match.group(2)}</a>'
        return xref
    else:
        return match.group(0)


def fix_footnotes(match):
    """
    Takes the <sup> footnotes and renders them more appropriately.
    """
    return f'<sup class="footnote">{match.group(1)}</sup>'



from lwm2pdf.clean_html import clean_html
from lwm2pdf.options import get_lwm2pdf_options
from lwm2pdf.process_options import (
        get_fn_information,
        get_output_information,
        get_stylesheet_from_options
        )
from lwm2pdf.process_pdf import create_pdf, open_pdf
from lwm2pdf.process_docs import markup_to_html
from lwm2pdf.filetypes import supported_file_types


def lwm2pdf(args=None):
    """
    This is the main function that takes us from CLI-provided
    arguments to a PDF
    """
    # setup
    options = get_lwm2pdf_options(supported_file_types, args)
    fn, fn_name = get_fn_information(options.input)
    output_fn, build_dir, output_html = get_output_information(options,
                                                               fn_name)
    css = get_stylesheet_from_options(options)
    styles = open(css, 'r').read()

    # processing
    html = markup_to_html(fn, supported_file_types)
    html = clean_html(html)

    # create output for checking/pasting to word
    open(output_html, 'w').write(html)
    print(f"Writing styled output to {fn_name}_styled...")
    html_with_styles = html.replace('</head>',
                                    f'<style>{styles}</style></head>')
    open(output_html.replace('.html', "_styled.html"), 'w').write(
                html_with_styles
        )

    if create_pdf(fn, output_fn, build_dir, output_html, styles, options):
        open_pdf(output_fn, options.ask_to_open)


if __name__ == '__main__':  # type: ignore
    lwm2pdf()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lwm2pdf']

package_data = \
{'': ['*'], 'lwm2pdf': ['themes/*', 'themes/source/*']}

install_requires = \
['asciidoc>=10.2.0,<11.0.0', 'markdown2>=2.4.3,<3.0.0', 'weasyprint==52.5']

entry_points = \
{'console_scripts': ['lwm2pdf = lwm2pdf.main:lwm2pdf']}

setup_kwargs = {
    'name': 'lwm2pdf',
    'version': '0.1.2',
    'description': 'Takes a variety of lighweight markup files and returns PDFs styled with provided css (via WeasyPrint)',
    'long_description': None,
    'author': 'Danny Elfanbaum',
    'author_email': 'drelfanbaum@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
